# TODO
# - allow disabling tests (currently bcond exists just for showing which are test deps)
# - add verbose build output (currently dummy bcond)
# - setup GOMAXPROCS=2 from _smp_mflags
# - fix CC containing spaces (ccache)
# - check if hg use at build time can be dropped
# - build all target archs, subpackage them: http://golang.org/doc/install/source#environment
#   or choose only useful crosscompilers?
# - subpackage -src files?

# Conditional build:
%bcond_without	verbose		# verbose build (V=1)
%bcond_without	tests		# build without tests [nop actually]
%bcond_without	shared		# Build golang shared objects for stdlib
%bcond_without	ext_linker	# Build golang using external/internal (close to cgo disabled) linking
%bcond_without	cgo		# cgo (importing C libraries) support
%bcond_with	bootstrap	# bootstrap build

%ifnarch %{ix86} %{x8664} %{arm} aarch64 mips64 mips64le ppc64le
%undefine	with_shared
%undefine	with_ext_linker
%undefine	with_cgo
%endif

Summary:	Go compiler and tools
Summary(pl.UTF-8):	Kompilator języka Go i narzędzia
Name:		golang
Version:	1.12.7
Release:	1
# source tree includes several copies of Mark.Twain-Tom.Sawyer.txt under Public Domain
License:	BSD and Public Domain
Group:		Development/Languages
# Source0Download: https://golang.org/dl/
Source0:	https://storage.googleapis.com/golang/go%{version}.src.tar.gz
# Source0-md5:	49d7a658cbd825f1cfe903d050bad29f
Patch0:		ca-certs.patch
Patch1:		0001-Don-t-use-the-bundled-tzdata-at-runtime-except-for-t.patch
URL:		http://golang.org/
BuildRequires:	bash
BuildRequires:	rpm-pythonprov
# The compiler is written in Go. Needs go(1.4+) compiler for build.
%if %{with bootstrap}
BuildRequires:	gcc-go >= 6:5
%else
BuildRequires:	golang >= 1.4
%endif
%if %{with tests}
BuildRequires:	glibc-static
BuildRequires:	hostname
BuildRequires:	pcre-devel
BuildRequires:	tzdata
%endif
Requires:	ca-certificates
Conflicts:	gcc-go
ExclusiveArch:	%{ix86} %{x8664} %{arm} aarch64 mips64 mips64le ppc64 ppc64le s390x
BuildRoot:	%{tmpdir}/%{name}-%{version}-root-%(id -u -n)

%define		no_install_post_strip	1
%define		no_install_post_chrpath	1
%define		_enable_debug_packages	0
%define		_noautoreqfiles		%{_libdir}/%{name}/src

%define	goroot %{_libdir}/%{name}

%ifarch %{ix86}
%define	GOARCH 386
%endif
%ifarch %{x8664}
%define	GOARCH amd64
%endif
%ifarch %{arm}
%define	GOARCH arm
%endif
%ifarch aarch64
%define	GOARCH arm64
%endif
%ifarch mips64
%define	GOARCH mips64x
%endif
%ifarch ppc64 ppc64le s390x
%define	GOARCH %{_arch}
%endif

%description
Go is an open source programming environment that makes it easy to
build simple, reliable, and efficient software.

%description -l pl.UTF-8
Go to mające otwarte źródła środowisko do programowania, pozwalające
na łatwe tworzenie prostych, pewnych i wydajnych programów.

%package shared
Summary:	Golang shared object libraries
Summary(pl.UTF-8):	Biblioteki obiektów współdzielonych dla języka Go
Group:		Libraries
Requires:	%{name} = %{version}-%{release}

%description shared
Golang shared object libraries.

%description shared -l pl.UTF-8
Biblioteki obiektów współdzielonych dla języka Go.

%package doc
Summary:	Documentation for Go language
Summary(fr.UTF-8):	Documentation pour Go
Summary(it.UTF-8):	Documentazione di Go
Summary(pl.UTF-8):	Dokumentacja do języka Go
Group:		Documentation
%if "%{_rpmversion}" >= "5"
BuildArch:	noarch
%endif

%description doc
Documentation for Go language.

%description doc -l fr.UTF-8
Documentation pour Go.

%description doc -l it.UTF-8
Documentazione di Go.

%description doc -l pl.UTF-8
Dokumentacja do języka Go.

%prep
%setup -qc
%{__mv} go/* .
%patch0 -p1
%patch1 -p1

# clean patch backups
find . -name '*.orig' | xargs -r %{__rm}

cat > env.sh <<'EOF'
# bootstrap compiler GOROOT
%if %{with bootstrap}
export GOROOT_BOOTSTRAP=%{_prefix}
%else
export GOROOT_BOOTSTRAP=%{goroot}
%endif
export GOROOT_FINAL=%{goroot}

export GOHOSTOS=linux
export GOHOSTARCH=%{GOARCH}

export GOOS=linux
export GOARCH=%{GOARCH}
%if %{without external_linker}
export GO_LDFLAGS="-linkmode internal"
%endif
%if %{with cgo}
export CGO_ENABLED=1
%else
export CGO_ENABLED=0
%endif

# use our gcc options for this build, but store gcc as default for compiler
export CFLAGS="%{rpmcflags}"
export LDFLAGS="%{rpmldflags}"

CC="%{__cc}"
export CC="${CC#ccache }"
export CC_FOR_TARGET="$CC"
EOF

%if 0
# optflags for go tools build
nflags="\"$(echo '%{rpmcflags}' | sed -e 's/^[ 	]*//;s/[ 	]*$//;s/[ 	]\+/ /g' -e 's/ /\",\"/g')\""
%{__sed} -i -e "s/\"-O2\"/$nflags/" src/cmd/dist/build.c
# NOTE: optflags used in gcc calls from go compiler are in src/cmd/go/build.go
%endif

%build
. ./env.sh
cd src
./make.bash --no-clean
cd ..

# build shared std lib
%if %{with shared}
GOROOT=$(pwd) PATH=$(pwd)/bin:$PATH go install -buildmode=shared std
%endif

%install
rm -rf $RPM_BUILD_ROOT
GOROOT=$RPM_BUILD_ROOT%{goroot}

install -d $GOROOT/{misc,lib,src}
install -d $RPM_BUILD_ROOT%{_bindir}

cp -a pkg lib bin src VERSION $GOROOT
cp -a misc/cgo $GOROOT/misc

# kill Win32 and Plan9 scripts
find $GOROOT -name '*.bat' -o -name '*.rc' | xargs %{__rm}

# https://github.com/golang/go/issues/4749
find $GOROOT/src | xargs touch -r $GOROOT/VERSION
# and level out all the built archives
touch $GOROOT/pkg
find $GOROOT/pkg | xargs touch -r $GOROOT/pkg

ln -sf %{_libdir}/%{name}/bin/go $RPM_BUILD_ROOT%{_bindir}/go
ln -sf %{_libdir}/%{name}/bin/gofmt $RPM_BUILD_ROOT%{_bindir}/gofmt
ln -sf %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/cgo $RPM_BUILD_ROOT%{_bindir}/cgo

# FIXME: do we need whole sources, including build scripts?
# for now, remove only non-Linux stuff
%{__rm} $RPM_BUILD_ROOT%{_libdir}/%{name}/src/androidtest.bash \
	$RPM_BUILD_ROOT%{_libdir}/%{name}/src/syscall/{mksyscall,mksysctl_openbsd,mksysnum_{darwin,dragonfly,freebsd,netbsd,openbsd}}.pl \
	$RPM_BUILD_ROOT%{_libdir}/%{name}/src/cmd/vendor/golang.org/x/sys/unix/{mksyscall_{aix_ppc,aix_ppc64,solaris},mksysctl_openbsd}.pl \
# ...and tests
%{__rm} -r $RPM_BUILD_ROOT%{_libdir}/%{name}/src/internal/trace \
	   $RPM_BUILD_ROOT%{_libdir}/%{name}/misc/cgo/{errors,fortran,test*}

# unenvize remaining scripts
%{__sed} -i -e '1s,/usr/bin/env bash,/bin/bash,' $RPM_BUILD_ROOT%{_libdir}/%{name}/src/*.bash
%{__sed} -i -e '1s,/usr/bin/env bash,/bin/bash,' $RPM_BUILD_ROOT%{_libdir}/%{name}/src/syscall/*.sh
%{__sed} -i -e '1s,/usr/bin/env bash,/bin/bash,' $RPM_BUILD_ROOT%{_libdir}/%{name}/src/cmd/vendor/golang.org/x/sys/unix/*.sh
%{__sed} -i -e '1s,/usr/bin/env bash,/bin/bash,' $RPM_BUILD_ROOT%{_libdir}/%{name}/src/cmd/go/*.sh
%{__sed} -i -e '1s,/usr/bin/env perl,/usr/bin/perl,' $RPM_BUILD_ROOT%{_libdir}/%{name}/src/syscall/*.pl

%clean
rm -rf $RPM_BUILD_ROOT

%files
%defattr(644,root,root,755)
%doc AUTHORS CONTRIBUTORS LICENSE
%attr(755,root,root) %{_bindir}/cgo
%attr(755,root,root) %{_bindir}/go
%attr(755,root,root) %{_bindir}/gofmt
%dir %{_libdir}/%{name}
%{_libdir}/%{name}/VERSION
%dir %{_libdir}/%{name}/bin
%attr(755,root,root) %{_libdir}/%{name}/bin/go
%attr(755,root,root) %{_libdir}/%{name}/bin/gofmt

%{_libdir}/%{name}/lib
%{_libdir}/%{name}/misc
%{_libdir}/%{name}/src
%dir %{_libdir}/%{name}/pkg
%{_libdir}/%{name}/pkg/linux_%{GOARCH}
%{_libdir}/%{name}/pkg/obj
%dir %{_libdir}/%{name}/pkg/tool
%dir %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/addr2line
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/api
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/asm
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/buildid
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/cgo
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/compile
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/cover
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/dist
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/doc
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/fix
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/link
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/nm
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/objdump
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/pack
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/pprof
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/test2json
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/trace
%attr(755,root,root) %{_libdir}/%{name}/pkg/tool/linux_%{GOARCH}/vet

%dir %{_libdir}/%{name}/pkg/bootstrap
%dir %{_libdir}/%{name}/pkg/bootstrap/bin
%attr(755,root,root) %{_libdir}/%{name}/pkg/bootstrap/bin/asm
%attr(755,root,root) %{_libdir}/%{name}/pkg/bootstrap/bin/cgo
%attr(755,root,root) %{_libdir}/%{name}/pkg/bootstrap/bin/compile
%attr(755,root,root) %{_libdir}/%{name}/pkg/bootstrap/bin/link
%{_libdir}/%{name}/pkg/bootstrap/pkg
%{_libdir}/%{name}/pkg/bootstrap/src
%{_libdir}/%{name}/pkg/include

%if %{with shared}
%files shared
%defattr(644,root,root,755)
%{_libdir}/%{name}/pkg/linux_%{GOARCH}_dynlink
%endif

%files doc
%defattr(644,root,root,755)
%doc doc/*
%changelog
* Fri Jul 26 2019 PLD Linux Team <feedback@pld-linux.org>
- For complete changelog see: http://git.pld-linux.org/?p=packages/golang.git;a=log;h=master

* Thu Jul 25 2019 Jan Palus <atler@pld-linux.org> 956ed35
- up to 1.12.7

* Mon Feb 18 2019 Jan Palus <atler@pld-linux.org> b0774e6
- up to 1.10.8 (fixes CVE-2019-6486)

* Fri Jan 11 2019 Jan Palus <atler@pld-linux.org> 83d213e
- up to 1.10.7 (fixes CVE-2018-16873, CVE-2018-16874, CVE-2018-16875)

* Thu Jul 19 2018 Jan Palus <atler@pld-linux.org> 2a683cc
- up to 1.10.3

* Thu Feb 08 2018 mis <mistoo@gmail.com> cd8e0a3
- up to 1.9.4 (CVE-2018-6574)

* Sat Jan 27 2018 Jan Palus <atler@pld-linux.org> c5489fe
- disable bootstrap by default

* Sat Jan 27 2018 Jan Palus <atler@pld-linux.org> 641563f
- up to 1.9.3

* Wed Oct 18 2017 Marcin Krol <hawk@tld-linux.org> 827674f
- fixed bootstrap build

* Wed May 31 2017 mis <mistoo@gmail.com> 34afcf9
- up to 1.8.3

* Sat Apr 01 2017 Jakub Bogusz <qboosh@pld-linux.org> 8022af9
- updated to 1.8
- removed outdated disable-TestGdbPython patch
- partial cleanup

* Wed Feb 22 2017 mis <mistoo@gmail.com> 7ce6963
- up to 1.7.5

* Wed Oct 12 2016 Jakub Bogusz <qboosh@pld-linux.org> d26aefa
- updated to 1.7.1
- updated disable-TestGdbPython patch

* Wed Oct 12 2016 Jakub Bogusz <qboosh@pld-linux.org> e541735
- pl for -shared,-doc
- removed obsolete binutils patch (R_386_GOT32X reloc and compressed debug sections seem supported now)

* Sun Sep 25 2016 Elan Ruusamäe <glen@delfi.ee> 4dc702d
- up to 1.6.3

* Mon Jun 13 2016 Elan Ruusamäe <glen@delfi.ee> b327c49
- shared subpackage, cgo bcond

* Mon Jun 13 2016 Elan Ruusamäe <glen@delfi.ee> ef64292
- up to 1.6.2; mmap-cgo-stackalign.patch applied upstream

* Wed Apr 27 2016 Elan Ruusamäe <glen@delfi.ee> 4567bac
- probably works, and if doesn't the project needs to be ported to 1.5+

* Tue Apr 05 2016 Elan Ruusamäe <glen@delfi.ee> c120016
- up to 1.6

* Tue Apr 05 2016 Elan Ruusamäe <glen@delfi.ee> 1602a4a
- go 1.4 dropped vim and emacs files

* Tue Apr 05 2016 Elan Ruusamäe <glen@delfi.ee> c21ac8b
- conflict with gcc-go

